package com.zswl.cloud.springBtach.server.core.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.github.microservice.auth.client.content.ResultContent;
import com.github.microservice.auth.client.content.ResultState;
import com.github.microservice.auth.security.helper.AuthHelper;
import com.zhongshu.payment.client.model.order.v2.CreatOrderResultModel;
import com.zhongshu.payment.client.model.order.v2.CreateOrder2Model;
import com.zhongshu.payment.client.service.v2.OrderService2;
import com.zhongshu.payment.client.type.JobFlowMap;
import com.zhongshu.payment.client.type.TransferOwner;
import com.zhongshu.payment.client.type.TransferType;
import com.zswl.cloud.shop.client.dto.life.TripartiteShopDto;
import com.zswl.cloud.shop.client.service.ShopService;
import com.zswl.cloud.shop.client.vo.life.ShopDetailVo;
import com.zswl.cloud.springBatch.client.model.dfy.Ticket.request.*;
import com.zswl.cloud.springBatch.client.model.dfy.Ticket.response.*;
import com.zswl.cloud.springBatch.client.service.DfyTicketService;
import com.zswl.cloud.springBtach.server.core.api.dfy.TicketApi;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

@Service
@Log4j2
public class DfyTicketServiceImpl implements DfyTicketService {

    @Autowired
    TicketApi ticketApi;

    @Autowired
    ShopService shopService;
    @Autowired
    private AuthHelper authHelper;

    @Autowired
    private Snowflake snowflake;

    @Autowired
    private OrderService2 orderService2;

    public ResultContent<ScenicListResponse> scenicList(ScenicListRequest scenicListRequest) {
        return ResultContent.buildContent(ticketApi.scenicList(scenicListRequest));
    }


    public ResultContent<ScenicDetailResponse> scenicDetail(ScenicDetailRequest scenicDetailRequest) {
        return ResultContent.buildContent(ticketApi.scenicDetail(scenicDetailRequest));
    }


    public ResultContent<PrivilegeTicketListResponse> privilegeTicketList(PrivilegeTicketListRequest privilegeTicketListRequest) {
        return ResultContent.buildContent(ticketApi.privilegeTicketList(privilegeTicketListRequest));
    }


    public ResultContent<DetailResponse> detail(DetailRequest detailRequest) {
        return ResultContent.buildContent(ticketApi.detail(detailRequest));
    }


    public ResultContent<CreatOrderResultModel> createOrderNew(CreateOrderNewRequest createOrderNewRequest) {

        String orderNo = snowflake.nextIdStr();
        createOrderNewRequest.setSourceOrderId(orderNo);

        DetailResponse detailResponse = ticketApi.detail(new DetailRequest(createOrderNewRequest.getProductId()));
        BigDecimal salePrice = detailResponse.getData().getSalePrice();
        BigDecimal payAmount = salePrice.subtract(BigDecimal.valueOf(createOrderNewRequest.getBookNumber()));

        ShopDetailVo shopDetailVo = shopService.detail(createOrderNewRequest.getEpId()).getContent();
        String authAccount = shopDetailVo.getAuthAccount();
        if (ObjectUtil.isEmpty(authAccount)) {
            return ResultContent.build(ResultState.Fail, "分账账户未设置");
        }
        CreateOrderNewRequest2 createOrderNewRequest2 = new CreateOrderNewRequest2();
        BeanUtils.copyProperties(createOrderNewRequest, createOrderNewRequest2);

        // 渠道订单
        CreateOrderNewResponse createOrderNewResponse = ticketApi.createOrderNew(createOrderNewRequest2);
        if (!createOrderNewResponse.getSuccess()) {
            return ResultContent.build(ResultState.Fail, createOrderNewResponse.getMsg());
        }

        // 平台订单
        CreateOrder2Model createOrder2Model = new CreateOrder2Model();
        createOrder2Model.setOrderNo(orderNo);
        createOrder2Model.setUserId(authHelper.getCurrentUser().getUserId());
        createOrder2Model.setShopId(createOrderNewRequest.getEpId());
        createOrder2Model.setShopInfo(JSONUtil.parseObj(shopDetailVo));
        createOrder2Model.setTotalAmount(payAmount);
        createOrder2Model.setDiscountAmount(BigDecimal.ZERO);
        createOrder2Model.setPayAmount(payAmount);
        createOrder2Model.setAccesserUserId(authAccount);
        createOrder2Model.setChannel(createOrder2Model.getChannel());

        // 商品信息
//        ArrayList<CreateOrder2Model.Goods> goodsArrayList = new ArrayList<>();
//        for (Integer i = 0; i < createOrderNewRequest.getBookNumber(); i++) {

        CreateOrder2Model.Goods goods = new CreateOrder2Model.Goods();
        goods.setGoodsId(String.valueOf(createOrderNewRequest.getProductId()));
        goods.setJobFlowMap(JobFlowMap.Ticket);
        goods.setGoodsInfo(JSONUtil.parseObj(detailResponse));
        goods.setPayAmount(payAmount);
        goods.setValidDay(30);
        goods.setTransferType(shopDetailVo.getShareType().equals(1) ? TransferType.RATIO : TransferType.FIXED);

        List<CreateOrder2Model.Owner> ownerList = new ArrayList<>();

        CreateOrder2Model.Owner platfor = new CreateOrder2Model.Owner();
        platfor.setOwner(TransferOwner.PLATFORM);
        platfor.setValue(shopDetailVo.getPlatformRate());

        CreateOrder2Model.Owner buy = new CreateOrder2Model.Owner();
        buy.setOwner(TransferOwner.BUY_SHOP);
        buy.setValue(shopDetailVo.getBuyShopRate());
        buy.setShopId(createOrderNewRequest.getEpId());

        ownerList.add(platfor);
        ownerList.add(buy);
        goods.setTransferRrule(ownerList);
        goods.setExtend(JSONUtil.toJsonStr(createOrderNewResponse.getData()));
//            goodsArrayList.add(goods);
//        }
        createOrder2Model.setGoodsList(List.of(goods));
        return orderService2.creat(createOrder2Model);
    }


    public ResultContent cancelOrder(CancelOrderRequest cancelOrderRequest) {
        ticketApi.cancelOrder(cancelOrderRequest);
        return ResultContent.build(ResultState.Success);
    }


    public ResultContent<SubmitOrderResponse> submitOrderResponse(SubmitOrderRequest submitOrderRequest) {
        return ResultContent.buildContent(ticketApi.submitOrderResponse(submitOrderRequest));

    }


    public ResultContent<OrderDetailResponse> orderDetail(OrderDetailRequest orderDetailRequest) {
        return ResultContent.buildContent(ticketApi.orderDetail(orderDetailRequest));

    }


    public ResultContent<OrderStatusResponse> orderStatus(OrderStatusRequest orderStatusRequest) {
        return ResultContent.buildContent(ticketApi.orderStatus(orderStatusRequest));

    }


    public ResultContent<VerifyOrderResponse> verifyOrder(VerifyOrderRequest verifyOrderRequest) {
        return ResultContent.buildContent(ticketApi.verifyOrder(verifyOrderRequest));

    }


    public ResultContent<TuiPiaoResponse> tuiPiao(TuiPiaoRequest tuiPiaoRequest) {
        return ResultContent.buildContent(ticketApi.tuiPiao(tuiPiaoRequest));

    }


    public void sync() {


        int pageNo = 1;
        int pageSize = 10;

        try {

            while (true) {
                ScenicListRequest scenicListRequest = new ScenicListRequest();
                scenicListRequest.setPage(pageNo);
                scenicListRequest.setPageSize(pageSize);

                ScenicListResponse scenicListResponse = ticketApi.scenicList(scenicListRequest);
                if (scenicListResponse.getData().getRows().size() == 0) {
                    return;
                }
                for (ScenicListResponse.rows row : scenicListResponse.getData().getRows()) {
                    ScenicDetailResponse scenicDetailResponse = ticketApi.scenicDetail(new ScenicDetailRequest(row.getScenicId()));
                    ScenicDetailResponse.Datas data = scenicDetailResponse.getData();

                    String glocation = row.getGlocation();
                    if (ObjectUtil.isEmpty(glocation)) {
                        break;
                    }
                    String[] split = glocation.split(",");

                    TripartiteShopDto tripartiteShopDto = new TripartiteShopDto();
                    tripartiteShopDto.setKey("门票");
                    tripartiteShopDto.setTripartiteId(row.getScenicId());
                    tripartiteShopDto.setShopName(row.getScenicName());
                    tripartiteShopDto.setShopMenuId("1");
                    tripartiteShopDto.setProvince(ObjectUtil.isEmpty(data.getProvinceName()) ? data.getCityName() : data.getProvinceName());
                    tripartiteShopDto.setCity(data.getCityName());
                    tripartiteShopDto.setAddress(row.getAddress());
                    tripartiteShopDto.setMapLon(split[1]);
                    tripartiteShopDto.setMapLat(split[0]);
                    tripartiteShopDto.setLogoPath(row.getNewPicUrl());
                    tripartiteShopDto.setDetailImg(data.getDefaultPic());
                    tripartiteShopDto.setTripartiteJson(JSONUtil.toJsonStr(data));

                    com.zswl.cloud.shop.client.ret.ResultContent resultContent = shopService.tripartiteShop(tripartiteShopDto);
                    log.info("景区创建店铺响应：{}", JSONUtil.toJsonStr(resultContent));


                }
                pageNo++;
            }

        } catch (Exception e) {
            log.error("pageNo：{}；pageSize：{}", pageNo, pageSize);
            e.printStackTrace();
        }


    }
}
